function [last_level] = amg_coarsen_plot(amg_grids, xy)
% AMG_COARSEN_PLOT      function for visualisation of amg coarsening
%
% last_level = amg_coarsen_plot(amg_grids, xy)
%
% Plots the points in a sequence of amg grids
%
% Input arguments
% ---------------
% amg_grids     generate using 'amg_grids_setup'
% xy            generated by IFISS, contains grid points
%               (if this optional argument is omitted, square regular mesh
%               is assumed)
%
% Output argument
% ---------------
% last_level(i) contains the value of the last level in which point i exists
%    PIFISS function: DJS; 31 January 2007. 
%  Copyright (c) 2007 by J. Boyle

n_levels = length(amg_grids);           % number of levels
n_points = length(amg_grids(1).A);      % number of points
last_level = ones(n_points,1);          % array to store last level for each point

level = n_levels;                       % start from last level
map = amg_grids(level).map_to_F;        % point i on level n corresponds to point map(i) on level n-1

while level > 1
    % update counts for C point in current level
    last_level(map) = last_level(map) + 1;

    if level > 2
        % get the map for the next level
        next_map = amg_grids(level-1).map_to_F;

        % map points in current level into new position on next level
        last_level_next = ones(n_points,1);
        last_level_next(next_map(map)) = last_level(map);

        last_level = last_level_next;
        map = next_map;
    end
    level = level - 1;
end

len = sqrt(n_points);
points = (1:n_points)';

if nargin < 2
    xy = zeros(n_points,2);
    xy(:,1) = -1 + 2*rem(points,len)/len;
    xy(:,2) = -1 + 2*floor(points/len)/len;
end


for level = 1:n_levels
    points = find(last_level >= level);

    figure(27)
    plot(xy(points, 1), xy(points,2),'+')
    axis([-1 1 -1 1])
    xlabel(num2str(level))
    if level < n_levels
        title('Press a key for next level')
        pause
    end
end
title('Finished')

